package cn.bunny.common.service.interceptor;

import cn.bunny.common.service.context.BaseContext;
import cn.bunny.common.service.utils.JwtHelper;
import cn.bunny.pojo.result.ResultCodeEnum;
import cn.bunny.pojo.result.constant.RedisUserConstant;
import cn.bunny.vo.system.login.LoginVo;
import com.alibaba.fastjson2.JSONObject;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import java.util.Map;

import static cn.bunny.common.service.utils.ResponseHandlerUtil.loginAuthHandler;

@Component
@Slf4j
public class UserTokenInterceptor implements HandlerInterceptor {

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        log.info("UserTokenInterceptor===>设置拦截器");
        // 获取token
        String token = request.getHeader("token");

        // token为空时
        if (token == null) {
            return loginAuthHandler(response, ResultCodeEnum.LOGIN_AUTH);
        }

        // 当token过期
        if (JwtHelper.isExpired(token)) {
            return loginAuthHandler(response, ResultCodeEnum.AUTHENTICATION_EXPIRED);
        }

        // 将token转成实体类
        Map<String, Object> tokenByMap = JwtHelper.getMapByToken(token);
        LoginVo loginVo = JSONObject.parseObject(JSONObject.toJSONString(tokenByMap), LoginVo.class);

        // 获取用户id和用户邮箱
        Long userId = loginVo.getId();
        String email = loginVo.getEmail();
        String redisKey = RedisUserConstant.getUserLoginInfoPrefix(email);
        Object redisUserinfo = redisTemplate.opsForValue().get(redisKey);

        // 不是动态方法直接返回
        if (!(handler instanceof HandlerMethod)) return true;

        // 解析不到userId，Redis中没有这个用户
        if (userId == null || redisUserinfo == null) {
            return loginAuthHandler(response, ResultCodeEnum.LOGIN_AUTH);
        }

        BaseContext.setUserId(userId);
        BaseContext.setUsername(email);
        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, @Nullable Exception ex) throws Exception {
        BaseContext.removeUser();
    }
}
